/*
	Joystick Wrapper Library (libjsw) Example Program

	To compile this program, type:

	cc -DJS_SUPPORT basic.c -o basic -ljsw


	This is a `basic' program using libjsw, it opens the joystick
	device using libjsw functions and handles recieved joystick
	axis and button values.

 */

#include <stdio.h>
#include <unistd.h>
#include <signal.h>

#ifdef JS_SUPPORT
/* Include joystick wrapper header file only if JS_SUPPORT is defined.
 * This is so that this program can be compiled without joystick
 * support. Though it would make it useless without joystick support,
 * we show this so that the programs you write can be compiled with
 * or without joystick support and still atleast `run' (and also help
 * with contrasting the portions of libjsw specific code used here).
 */
# include <jsw.h>
#endif	/* JS_SUPPORT */


/*
 *	Global runlevel code for this program, used for easy start and
 *	exiting of the main while() loop.
 */
static int runlevel;


/*
 *	Signal handler, to catch the interrupt (ctrl+c) signal.
 */
static void SignalHandler(int s)
{
	switch(s)
	{
	  case SIGINT:
	    /* Set runlevel to 1, causing the main while() loop to break. */
	    runlevel = 1;
	    break;
	}

	return;
}


int main(int argc, char *argv[])
{
	int i, status;
	char *device = "/dev/js0";	/* Device file name for the joystick. */
	char *calib = ".joystick";	/* Calibration file. */
#ifdef JS_SUPPORT
	js_data_struct jsd;	/* Structure that contains the referance
				 * to the opened joystick. If you plan to
				 * open more joysticks you need a jsd
				 * structure for each one.
				 */
#endif	/* JS_SUPPORT */


	/* Set up signal handler to catch interrupt signal. */
	signal(SIGINT, SignalHandler);


	/* Parse arguments, first argument is the joystick device name
	 * and the second is the calibration file name. Did we get
	 * enough arguments?
	 */
	if(argc < 3)
	{
	    /* Not enough arguments given, print usage and exit. */
	    printf(
		"Usage: basic <joystick_device> <calibation_file>\n"
	    );
	    return(0);
	}

	/* Fetch device and calibration file names from input. */
	device = argv[1];
	calib = argv[2];

#ifdef JS_SUPPORT
	/* Initialize (open) the joystick, passing it the pointer to
	 * the jsd structure (it is safe to pass an uninitialized jsd
	 * structure as seen here).
	 */
	status = JSInit(
	    &jsd,
	    device,
	    calib,
	    JSFlagNonBlocking
	);
	/* Error occured opening joystick? */
	if(status != JSSuccess)
	{
	    /* There was an error opening the joystick, print by the
	     * recieved error code from JSInit().
	     */
	    switch(status)
	    {
	      case JSBadValue:
		fprintf(stderr,
"%s: Invalid value encountered while initializing joystick.\n",
		    device
		);
		break;

	      case JSNoAccess:
		fprintf(stderr,
"%s: Cannot access resources, check permissions and file locations.\n",
                    device
                );
		break;

	      case JSNoBuffers:
                fprintf(stderr,
"%s: Insufficient memory to initialize.\n",
                    device
                );
		break;

	      default:	/* JSError */
                fprintf(stderr,
"%s: Error occured while initializing.\n",
                    device
                );
		break;
	    }

	    /* Print some helpful advice. */
	    fprintf(stderr,
"Make sure that:\n\
    1. Your joystick is connected (and turned on as needed).\n\
    2. Your joystick modules are loaded (`modprobe <driver>').\n\
    3. Your joystick needs to be calibrated (use `jscalibrator').\n\
\n"
            );

	    /* Close joystick jsd structure (just in case). */
	    JSClose(&jsd);

	    return(1);
	}
	else
	{
	    /* Opened joystick successfully. */
	    printf(
"Reading values from `%s' (press ctrl+c to interrupt).\n",
		device
	    );
	}
#endif	/* JS_SUPPORT */


	/* Set runlevel to 2 and begin the main while() loop. Here we
	 * will print the values of each axis and button as they
	 * change.
	 */
	runlevel = 2;
	while(runlevel >= 2)
	{
#ifdef JS_SUPPORT
	    /* Fetch for event, JSUpdate() will return JSGotEvent
	     * if we got an event.
	     */
	    if(JSUpdate(&jsd) == JSGotEvent)
	    {
		/* Got an event, now print the values to reflect the
		 * change in axises and/or buttons.
		 */
		printf("\r");

		/* Print value of each axis. */
		for(i = 0; i < jsd.total_axises; i++)
		    printf(
			"A%i:%.3f ",
			i,
			JSGetAxisCoeffNZ(&jsd, i)
		    );

		printf(" ");

		/* Print state of each button. */
		for(i = 0; i < jsd.total_buttons; i++)
                    printf(
                        "B%i:%i ",
			i,
                        JSGetButtonState(&jsd, i)
                    );

		/* That's it, besure to flush output stream. */
		fflush(stdout);
	    }
#endif  /* JS_SUPPORT */

	    usleep(16000);	/* Don't hog the cpu. */
	}


#ifdef JS_SUPPORT
	/* Close the joystick, now jsd may no longer be used after
	 * this call to any other JS*() calls except JSInit() which
	 * would initialize it again.
	 */
	JSClose(&jsd);
#endif  /* JS_SUPPORT */


	return(0);
}
